#include "forms.h"
#include "xfpovray.h"
#include "pov-defaults.h"

extern struct pov_options pov;

extern char *POV_INIFILE;
extern char *STREAM_FILE;
extern char *FATAL_FILE;

extern char *CONFIG_POV_WIDTH; 
extern char *CONFIG_POV_HEIGHT;

extern XFDATA_S *xfData;
extern PARSING_PAGE_S   *parsingPage;
extern OUTPUT_PAGE_S	*outputPage;
extern TRACING_PAGE_S   *tracingPage;
extern ANIMATION_PAGE_S *animationPage;
extern CONFIG_PAGE_S	*configPage;

 
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* 
** Function to write the povray configuration file 
*/
void write_config(void)
{
    FILE *options_file;
    int i;
   
              
    if ((options_file = fopen(POV_INIFILE, "w")) == NULL) {
       printf("xfpovray: write_config\n");
       printf("  Could not write config file %s\n", POV_INIFILE);
       exit (1);
    }



    fprintf(options_file, "version=%3.1f\n", (float) pov.version);    

    fprintf(options_file, "input_file_name=%s\n", pov.input_file_name);
    
    /* Library path */
    for (i=0; i<5; i++) {
        if (strcmp(fl_get_input (parsingPage->libpathInput[i]), "") != 0)
           fprintf(options_file, "library_path=%s\n", 
               fl_get_input (parsingPage->libpathInput[i]));
    }
               
    /* Image height and width. 
    ** Must parse the input boxes in case the user 
    ** made an entry by clicking in the input box instead
    ** of using the pulldown menu.
    */
    if (strcmp(fl_get_input (outputPage->imageHeightInput), "") == 0) {
       fl_set_input (outputPage->imageHeightInput, CONFIG_POV_HEIGHT);
       pov.height = atoi(CONFIG_POV_HEIGHT);
    }
    else
       pov.height = atoi(fl_get_input (outputPage->imageHeightInput));

    if (strcmp(fl_get_input (outputPage->imageWidthInput), "") == 0) {
       fl_set_input (outputPage->imageWidthInput, CONFIG_POV_WIDTH);
       pov.width = atoi(CONFIG_POV_WIDTH);
    }
    else
       pov.width = atoi(fl_get_input (outputPage->imageWidthInput)); 
       
    fprintf(options_file, "height=%d\n", pov.height); 
    fprintf(options_file, "width=%d\n", pov.width); 

    fprintf(options_file, "verbose=%d\n", pov.verbose);

    fprintf(options_file, "buffer_output=%d\n", pov.buffer_output);
    if (pov.buffer_output)
       fprintf(options_file, "buffer_size=%d\n", pov.buffer_size);

    /* Redirection options.                   
    ** Turn off redirect to console; redirect to file if selected,
    ** then display file in message browser.
    */
    fprintf(options_file, "all_console=%d\n", 0);
    if (pov.redirect_on == 1)
       fprintf(options_file, "all_file=%s\n", STREAM_FILE); 
    else
       fprintf(options_file, "fatal_file=%s\n", FATAL_FILE);
 
    /* Partial image height and width; parse the input boxes */ 
    if (strcmp(fl_get_input (outputPage->columnStart), "") != 0)
       fprintf(options_file, "start_column=%s\n", 
       fl_get_input (outputPage->columnStart)); 
    if (strcmp(fl_get_input (outputPage->columnEnd), "") != 0)
       fprintf(options_file, "end_column=%s\n", 
       fl_get_input (outputPage->columnEnd)); 
    if (strcmp(fl_get_input (outputPage->rowStart), "") != 0)
       fprintf(options_file, "start_row=%s\n", 
       fl_get_input (outputPage->rowStart)); 
    if (strcmp(fl_get_input (outputPage->rowEnd), "") != 0)
       fprintf(options_file, "end_row=%s\n", 
       fl_get_input (outputPage->rowEnd)); 

    /* Mosaic preview; parse the input boxes */
    if (strcmp(fl_get_input (outputPage->mosaicStart), "") != 0)
       fprintf(options_file, "preview_start_size=%s\n", 
               fl_get_input (outputPage->mosaicStart)); 
    if (strcmp(fl_get_input (outputPage->mosaicEnd), "") != 0)
       fprintf(options_file, "preview_end_size=%s\n", 
               fl_get_input (outputPage->mosaicEnd)); 
 
  
    fprintf(options_file, "quality=%d\n", pov.quality); 

    fprintf(options_file, "antialias=%d\n", pov.antialias); 
    fprintf(options_file, "antialias_depth=%d\n", pov.aa_depth); 
    fprintf(options_file, "sampling_method=%d\n", pov.aa_sampling_method); 
    fprintf(options_file, "antialias_threshold=%s\n", 
        fl_get_input (tracingPage->antialiasThreshold)); 

    fprintf(options_file, "radiosity=%d\n", pov.radiosity); 

    fprintf(options_file, "display=%d\n", pov.display_during_render);
        
    /* If displaying during a render, pause to view image */    
    if (pov.display_during_render == 1)
       fprintf(options_file, "pause_when_done=%d\n", 1); 

    fprintf(options_file, "output_to_file=%d\n", pov.output_to_file);

    fprintf(options_file, "remove_bounds=%d\n", pov.remove_bounds);

    fprintf(options_file, "split_unions=%d\n", pov.split_unions); 

    fprintf(options_file, "bounding=%d\n", pov.bounding);
    
    if (strcmp(fl_get_input (tracingPage->boundingThreshold), "") != 0)
       pov.bounding_threshold = 
           atoi(fl_get_input (tracingPage->boundingThreshold));
    else 
       pov.bounding_threshold = POV_BOUNDING_THRESHOLD; 
    fprintf(options_file, "bounding_threshold=%d\n", pov.bounding_threshold); 

    fprintf(options_file, "light_buffer=%d\n", pov.light_buffer);

    fprintf(options_file, "vista_buffer=%d\n", pov.vista_buffer);

    fprintf(options_file, "clock=%s\n", 
        fl_get_input (animationPage->clockExternalInput)); 

    fprintf(options_file, "initial_clock=%s\n", 
        fl_get_input (animationPage->clockStartInput)); 

    fprintf(options_file, "final_clock=%s\n", 
        fl_get_input (animationPage->clockEndInput)); 
 
    pov.initial_frame = atoi(fl_get_input (animationPage->frameStartInput));
    fprintf(options_file, "initial_frame=%d\n", pov.initial_frame);
      
    pov.final_frame = atoi(fl_get_input (animationPage->frameEndInput));
    fprintf(options_file, "final_frame=%d\n", pov.final_frame);

    if (strcmp(fl_get_input (animationPage->subsetFrameStartInput), "") != 0) {
       fprintf(options_file, "subset_start_frame=%s\n", 
           fl_get_input (animationPage->subsetFrameStartInput));
    }
    if (strcmp(fl_get_input (animationPage->subsetFrameEndInput), "") != 0) {
       fprintf(options_file, "subset_end_frame=%s\n", 
           fl_get_input (animationPage->subsetFrameEndInput));
    }
       
    fprintf(options_file, "cyclic_animation=%d\n", pov.cyclic_animation);

    fprintf(options_file, "field_render=%d\n", pov.field_rendering);
    if (pov.field_rendering == 1)
       fprintf(options_file, "odd_field=%d\n", pov.odd_field);

    switch (pov.palette) {
           case 1: fprintf(options_file, "palette=T\n");
                   break;
           case 2: fprintf(options_file, "palette=H\n");
                   break;
           case 3: fprintf(options_file, "palette=G\n");
                   break;
    }           

    switch (pov.output_filetype) {
           case 1: fprintf(options_file, "output_file_type=T\n");
                   break;
           case 2: fprintf(options_file, "output_file_type=C\n");
                   break;
           case 3: fprintf(options_file, "output_file_type=N\n");
                   break;
           case 4: fprintf(options_file, "output_file_type=P\n");
                   break;
           case 5: fprintf(options_file, "output_file_type=S\n");
                   break;
    }           

    fprintf(options_file, "jitter=%d\n", pov.jitter); 

    fprintf(options_file, "jitter_amount=%s\n", 
        fl_get_input (tracingPage->jitterInput)); 


    fclose(options_file);

    /*
    ** Update the config folder
    */
    fl_freeze_form (configPage->form);
    fl_clear_browser (configPage->configBrowser);
    fl_load_browser (configPage->configBrowser, POV_INIFILE);    
    fl_unfreeze_form (configPage->form);
    
 
}
